#!/usr/bin/env python

"""XCP info/exploration tool.
"""

from pprint import pprint

from pyxcp.cmdline import ArgumentParser
from pyxcp.types import TryCommandResult


def main():
    ap = ArgumentParser(description="XCP info/exploration tool.")

    with ap.run() as x:
        x.connect()
        if x.slaveProperties.optionalCommMode:
            x.try_command(x.getCommModeInfo, extra_msg="availability signaled by CONNECT, this may be a slave configuration error.")
        print("\nSlave Properties:")
        print("=================")
        pprint(x.slaveProperties)

        result = x.id_scanner()
        print("\n")
        print("Implemented IDs:")
        print("================")
        for key, value in result.items():
            print(f"{key}: {value}", end="\n\n")
        cps = x.getCurrentProtectionStatus()
        print("\nProtection Status")
        print("=================")
        for k, v in cps.items():
            print(f"    {k:6s}: {v}")
        x.cond_unlock()
        print("\nDAQ Info:")
        print("=========")
        daq_info = x.getDaqInfo()
        pprint(daq_info)

        daq_pro = daq_info["processor"]
        daq_properties = daq_pro["properties"]
        if x.slaveProperties.transport_layer == "CAN":
            print("")
            if daq_properties["pidOffSupported"]:
                print("*** pidOffSupported -- i.e. one CAN-ID per DAQ-list.")
            else:
                print("*** NO support for PID_OFF")
        num_predefined = daq_pro["minDaq"]
        print("\nPredefined DAQ-Lists")
        print("====================")
        if num_predefined > 0:
            print(f"There are {num_predefined} predefined DAQ-lists")
            for idx in range(num_predefined):
                print(f"DAQ-List #{idx}\n____________\n")
                status, dm = x.try_command(x.getDaqListMode, idx)
                if status == TryCommandResult.OK:
                    print(dm)
                status, di = x.try_command(x.getDaqListInfo, idx)
                if status == TryCommandResult.OK:
                    print(di)
        else:
            print("*** NO Predefined DAQ-Lists")
        print("\nPAG Info:")
        print("=========")
        if x.slaveProperties.supportsCalpag:
            status, pag = x.try_command(x.getPagProcessorInfo)
            if status == TryCommandResult.OK:
                print(pag)
                # for idx in range(pag.maxSegments):
                #     x.getSegmentInfo(0x01, idx, 0, 0)
        else:
            print("*** PAGING IS NOT SUPPORTED.")

        print("\nPGM Info:")
        print("=========")
        if x.slaveProperties.supportsPgm:
            status, pgm = x.try_command(x.getPgmProcessorInfo)
            if status == TryCommandResult.OK:
                print(pgm)
        else:
            print("*** FLASH PROGRAMMING IS NOT SUPPORTED.")

        if x.slaveProperties.transport_layer == "CAN":
            print("\nTransport-Layer CAN:")
            print("====================")
            status, res = x.try_command(x.getSlaveID, 0)
            if status == TryCommandResult.OK:
                print("CAN identifier for CMD/STIM:\n", res)
            else:
                print("*** GET_SLAVE_ID() IS NOT SUPPORTED.")  # no response from bc address ???

            print("\nPer DAQ-list Identifier")
            print("-----------------------")
            daq_id = 0
            while True:
                status, res = x.try_command(x.getDaqId, daq_id)
                if status == TryCommandResult.OK:
                    print(f"DAQ-list #{daq_id}:", res)
                    daq_id += 1
                else:
                    break
            if daq_id == 0:
                print("N/A")
        x.disconnect()
        print("\nDone.")


if __name__ == "__main__":
    main()



from sys import version_info

from pyxcp.utils import PYTHON_VERSION, flatten, getPythonVersion, hexDump, slicer


def test_hexdump(capsys):
    print(hexDump(range(16)), end="")
    captured = capsys.readouterr()
    assert captured.out == "[00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e 0f]"


def test_slicer1():
    res = slicer([1, 2, 3, 4, 5, 6, 7, 8], 4)
    assert res == [[1, 2, 3, 4], [5, 6, 7, 8]]


def test_slicer2():
    res = slicer(["10", "20", "30", "40", "50", "60", "70", "80"], 4, tuple)
    assert res == [("10", "20", "30", "40"), ("50", "60", "70", "80")]


def test_flatten1():
    res = flatten([[1, 2, 3, 4], [5, 6, 7, 8]])
    assert res == [1, 2, 3, 4, 5, 6, 7, 8]


def test_version():
    assert getPythonVersion() == version_info
    assert PYTHON_VERSION == version_info
    assert getPythonVersion() == PYTHON_VERSION



import pytest

from pyxcp import checksum


"""
XCP_ADD_11          0x10        0x10
XCP_ADD_12          0x0F10      0x0F10
XCP_ADD_14          0x00000F10  0x00000F10
XCP_ADD_22          0x1800      0x0710
XCP_ADD_24          0x00071800  0x00080710
XCP_ADD_44          0x140C03F8  0xFC040B10

XCP_CRC_16          0xC76A      0xC76A
XCP_CRC_16_CITT     0x9D50      0x9D50
XCP_CRC_32          0x89CD97CE  0x89CD97CE
"""

TEST = bytes(
    (
        0x01,
        0x02,
        0x03,
        0x04,
        0x05,
        0x06,
        0x07,
        0x08,
        0x09,
        0x0A,
        0x0B,
        0x0C,
        0x0D,
        0x0E,
        0x0F,
        0x10,
        0xF1,
        0xF2,
        0xF3,
        0xF4,
        0xF5,
        0xF6,
        0xF7,
        0xF8,
        0xF9,
        0xFA,
        0xFB,
        0xFC,
        0xFD,
        0xFE,
        0xFF,
        0x00,
    )
)


def testAdd11():
    assert checksum.check(TEST, "XCP_ADD_11") == 0x10


def testAdd12():
    assert checksum.check(TEST, "XCP_ADD_12") == 0x0F10


def testAdd14():
    assert checksum.check(TEST, "XCP_ADD_14") == 0x00000F10


def testAdd22():
    assert checksum.check(TEST, "XCP_ADD_22") == 0x1800


def testAdd24():
    assert checksum.check(TEST, "XCP_ADD_24") == 0x00071800


def testAdd44():
    assert checksum.check(TEST, "XCP_ADD_44") == 0x140C03F8


def testCrc16():
    assert checksum.check(TEST, "XCP_CRC_16") == 0xC76A


def testCrc16Ccitt():
    assert checksum.check(TEST, "XCP_CRC_16_CITT") == 0x9D50


def testCrc32():
    assert checksum.check(TEST, "XCP_CRC_32") == 0x89CD97CE


def testUserDefined():
    with pytest.raises(NotImplementedError):
        checksum.check(TEST, "XCP_USER_DEFINED")



from collections import defaultdict

from traitlets.config import LoggingConfigurable
from traitlets.config.loader import Config


LEGACY_KEYWORDS = {
    # General
    "LOGLEVEL": "General.loglevel",
    "DISABLE_ERROR_HANDLING": "General.disable_error_handling",
    "SEED_N_KEY_DLL": "General.seed_n_key_dll",
    "SEED_N_KEY_DLL_SAME_BIT_WIDTH": "General.seed_n_key_dll_same_bit_width",
    "DISCONNECT_RESPONSE_OPTIONAL": "General.disconnect_response_optional",
    # Transport
    "TRANSPORT": "Transport.layer",
    "CREATE_DAQ_TIMESTAMPS": "Transport.create_daq_timestamps",
    "TIMEOUT": "Transport.timeout",
    "ALIGNMENT": "Transport.alignment",
    # Eth
    "HOST": "Transport.Eth.host",
    "PORT": "Transport.Eth.port",
    "PROTOCOL": "Transport.Eth.protocol",
    "IPV6": "Transport.Eth.ipv6",
    "TCP_NODELAY": "Transport.Eth.tcp_nodelay",
    # Usb
    "SERIAL_NUMBER": "Transport.Usb.serial_number",
    "CONFIGURATION_NUMBER": "Transport.Usb.configuration_number",
    "INTERFACE_NUMBER": "Transport.Usb.interface_number",
    "COMMAND_ENDPOINT_NUMBER": "Transport.Usb.out_ep_number",
    "REPLY_ENDPOINT_NUMBER": "Transport.Usb.in_ep_number",
    "VENDOR_ID": "Transport.Usb.vendor_id",
    "PRODUCT_ID": "Transport.Usb.product_id",
    "LIBRARY": "Transport.Usb.library",
    # Can
    "CAN_DRIVER": "Transport.Can.interface",
    "CHANNEL": "Transport.Can.channel",
    "MAX_DLC_REQUIRED": "Transport.Can.max_dlc_required",
    #   "MAX_CAN_FD_DLC": "Transport.Can.max_can_fd_dlc",
    "PADDING_VALUE": "Transport.Can.padding_value",
    "CAN_USE_DEFAULT_LISTENER": "Transport.Can.use_default_listener",
    # Swap master and slave IDs. (s. https://github.com/christoph2/pyxcp/issues/130)
    "CAN_ID_SLAVE": "Transport.Can.can_id_master",
    "CAN_ID_MASTER": "Transport.Can.can_id_slave",
    "CAN_ID_BROADCAST": "Transport.Can.can_id_broadcast",
    "BITRATE": "Transport.Can.bitrate",
    "RECEIVE_OWN_MESSAGES": "Transport.Can.receive_own_messages",
    "POLL_INTERVAL": "Transport.Can.poll_interval",
    "FD": "Transport.Can.fd",
    "DATA_BITRATE": "Transport.Can.data_bitrate",
    "ACCEPT_VIRTUAL": "Transport.Can.Kvaser.accept_virtual",
    "SJW": "Transport.Can.sjw_abr",
    "TSEG1": "Transport.Can.tseg1_abr",
    "TSEG2": "Transport.Can.tseg2_abr",
    "TTY_BAUDRATE": "Transport.Can.SlCan.ttyBaudrate",
    "UNIQUE_HARDWARE_ID": "Transport.Can.Ixxat.unique_hardware_id",
    "RX_FIFO_SIZE": "Transport.Can.Ixxat.rx_fifo_size",
    "TX_FIFO_SIZE": "Transport.Can.Ixxat.tx_fifo_size",
    "DRIVER_MODE": "Transport.Can.Kvaser.driver_mode",
    "NO_SAMP": "Transport.Can.Kvaser.no_samp",
    "SINGLE_HANDLE": "Transport.Can.Kvaser.single_handle",
    "USE_SYSTEM_TIMESTAMP": "Transport.Can.Neovi.use_system_timestamp",
    "OVERRIDE_LIBRARY_NAME": "Transport.Can.Neovi.override_library_name",
    "BAUDRATE": "Transport.Can.Serial.baudrate",
    "SLEEP_AFTER_OPEN": "Transport.Can.SlCan.sleep_after_open",
    "DEVICE_NUMBER": "Transport.Can.Systec.device_number",
    "RX_BUFFER_ENTRIES": "Transport.Can.Systec.rx_buffer_entries",
    "TX_BUFFER_ENTRIES": "Transport.Can.Systec.tx_buffer_entries",
    "FLAGS": "Transport.Can.Usb2Can.flags",
    "APP_NAME": "Transport.Can.Vector.app_name",
    "RX_QUEUE_SIZE": "Transport.Can.Vector.rx_queue_size",
}


def nested_dict_update(d: dict, key: str, value) -> None:
    root, *path, key = key.split(".")
    sub_dict = d[root]
    for part in path:
        if part not in sub_dict:
            sub_dict[part] = defaultdict(dict)
        sub_dict = sub_dict[part]
    sub_dict[key] = value


def convert_config(legacy_config: dict, logger: LoggingConfigurable) -> Config:
    interface_name = None
    resolv = []
    d = defaultdict(dict)
    for key, value in legacy_config.items():
        key = key.upper()
        item = LEGACY_KEYWORDS.get(key)
        if item is None:
            logger.warning(f"Unknown keyword {key!r} in config file")
            continue
        if key == "CAN_DRIVER":
            value = value.lower()
            interface_name = value
        if key in ("SERIAL", "LOG_ERRORS", "STATE", "RTSCTS"):
            resolv.append((key, value))
        else:
            nested_dict_update(d=d, key=item, value=value)
    for key, value in resolv:
        if key == "SERIAL":
            if interface_name == "neovi":
                d["Transport.Can.Neovi.serial"] = value
            elif interface_name == "vector":
                d["Transport.Can.Vector.serial"] = value
        elif key == "LOG_ERRORS":
            if interface_name == "nican":
                d["Transport.Can.NiCan.log_errors"] = value
        elif key == "STATE":
            if interface_name == "pcan":
                d["Transport.Can.PCan.state"] = value
            elif interface_name == "systec":
                d["Transport.Can.Systec.state"] = value
        elif key == "RTSCTS":
            if interface_name == "serial":
                d["Transport.Can.Serial.rtscts"] = value
            elif interface_name == "slcan":
                d["Transport.Can.SlCan.rtscts"] = value
    return Config(d)
